# pylint: disable=no-self-use,unused-argument

"""Unit tests configuration file."""


import datafiles
import log
import pytest

from pomace import shared


class MockLinks:
    def find_by_partial_text(self, value):
        return [f'<mockelement: links.partial_text={value}>']


class MockBrowser:

    url = 'http://example.com'

    html = "Hello, world!"

    def find_by_name(self, value):
        return [f'<mockelement: name={value}>']

    links = MockLinks()


@pytest.fixture
def mockbrowser(monkeypatch):
    monkeypatch.setattr(shared, 'browser', MockBrowser())


def pytest_configure(config):
    log.init(debug=True)

    terminal = config.pluginmanager.getplugin('terminal')
    terminal.TerminalReporter.showfspath = False


def pytest_runtest_setup(item):
    datafiles.settings.HOOKS_ENABLED = False



# pylint: disable=expression-not-assigned,unused-variable,redefined-outer-name,unused-argument


from pomace import cli, config


def describe_prompt_for_browser_if_unset():
    def when_ci(expect, monkeypatch):
        monkeypatch.setenv('CI', 'true')
        config.settings.browser.name = ''
        cli.prompt_for_browser_if_unset()
        expect(config.settings.browser.name) == 'firefox'

    def when_ci_and_override(expect, monkeypatch):
        monkeypatch.setenv('CI', 'true')
        monkeypatch.setenv('BROWSER', 'chrome')
        config.settings.browser.name = ''
        cli.prompt_for_browser_if_unset()
        expect(config.settings.browser.name) == 'chrome'


def describe_prompt_for_url_if_unset():
    def when_ci(expect, monkeypatch, mockbrowser):
        monkeypatch.setenv('CI', 'true')
        config.settings.url = ''
        cli.prompt_for_url_if_unset()
        expect(config.settings.url) == 'http://example.com'


def describe_prompt_for_secret_if_unset():
    def when_ci(expect, monkeypatch, mockbrowser):
        monkeypatch.setenv('CI', 'true')
        cli.prompt_for_secret_if_unset('foobar')
        expect(config.settings.get_secret('foobar')) == '<unset>'



from dataclasses import dataclass
from typing import Optional
from urllib.parse import urlparse

from parse import parse


__all__ = ['URL']


class URL:

    ROOT = '@'

    def __init__(self, url_or_domain: str, path: Optional[str] = None):
        if path == self.ROOT:
            self.value = f'https://{url_or_domain}'
        elif path:
            path = ('/' + path).rstrip('/').replace('//', '/')
            self.value = f'https://{url_or_domain}' + path
        else:
            self.value = str(url_or_domain).rstrip('/')

    def __repr__(self):
        return f"URL({self.value!r})"

    def __str__(self):
        return self.value

    def __eq__(self, other):
        if not isinstance(other, self.__class__):
            return str(self) == str(other).strip('/')

        if self.domain != other.domain:
            return False
        if self.path == other.path:
            return True

        result = parse(self.path, other.path)
        if not result:
            return False
        for value in result.named.values():
            if '/' in value:
                return False

        return True

    def __ne__(self, other):
        return not self.__eq__(other)

    def __contains__(self, value):
        return value in self.value

    @property
    def domain(self) -> str:
        return urlparse(self.value).netloc

    @property
    def path(self) -> str:
        path = urlparse(self.value).path.strip('/')
        return path if path else self.ROOT

    @property
    def fragment(self) -> str:
        return urlparse(self.value).fragment.replace('/', '_').strip('_')


@dataclass
class Person:
    honorific: str
    first_name: str
    last_name: str
    email_address: str
    zip_code: str

    @property
    def prefix(self) -> str:
        return self.honorific

    @property
    def email(self) -> str:
        return self.email_address

    @property
    def zip(self) -> str:
        return self.zip_code



from pkg_resources import DistributionNotFound, get_distribution

from .api import *
from .models import *
from .types import *


try:
    __version__ = get_distribution('pomace').version
except DistributionNotFound:
    __version__ = '(local)'



# pylint: disable=no-self-use

import os
from importlib import import_module, reload

import log
from cleo import Application, Command
from IPython import embed

from . import browser, models, utils
from .config import settings


try:
    import bullet
except ImportError:
    bullet = None  # https://github.com/Mckinsey666/bullet/issues/2
    log.warn("Interactive CLI prompts not yet supported on Windows")


def prompt_for_browser_if_unset():
    if settings.browser.name:
        return

    if 'CI' in os.environ or not bullet:
        settings.browser.name = os.getenv('BROWSER', 'firefox')
        return

    cli = bullet.Bullet(
        prompt="\nSelect a browser for automation: ",
        bullet=" ● ",
        choices=browser.NAMES,
    )
    settings.browser.name = cli.launch().lower()


def prompt_for_url_if_unset():
    if settings.url:
        return

    if 'CI' in os.environ or not bullet:
        settings.url = 'http://example.com'
        return

    domains = [p.domain for p in models.Page.objects.all()]
    if domains:
        cli = bullet.Bullet(
            prompt="\nStarting domain: ", bullet=" ● ", choices=list(set(domains))
        )
    else:
        cli = bullet.Input(prompt="\nStarting domain: ", strip=True)
    domain = cli.launch()
    settings.url = f"https://{domain}"


def prompt_for_secret_if_unset(name: str):
    if settings.get_secret(name, _log=False):
        return

    if 'CI' in os.environ or not bullet:
        settings.set_secret(name, '<unset>')
        return

    cli = bullet.Input(prompt=f"{name}: ")
    value = cli.launch()
    settings.set_secret(name, value)


class BaseCommand(Command):
    def handle(self):
        log.reset()
        log.init(verbosity=self.io.verbosity + 1)
        log.silence('datafiles', allow_warning=True)
        self.update_settings()
        prompt_for_browser_if_unset()
        prompt_for_url_if_unset()
        utils.launch_browser()
        utils.locate_models()
        try:
            self.run_loop()
        finally:
            utils.quit_browser()
            print()

    def update_settings(self):
        if self.option("root"):
            os.chdir(self.option("root"))

        if self.option("browser"):
            settings.browser.name = self.option("browser").lower()

        settings.browser.headless = self.option("headless")

        if self.option("domain"):
            settings.url = "https://" + self.option("domain")


class ShellCommand(BaseCommand):
    """
    Launch an interactive shell

    shell
        {--browser= : Browser to use for automation}
        {--headless : Run the specified browser in a headless mode}
        {--domain= : Starting domain for the automation}
        {--root= : Directory to load models from}
    """

    def run_loop(self):
        # pylint: disable=unused-variable
        pomace = import_module('pomace')
        Page = models.Page
        autopage = models.autopage
        page = autopage()
        embed(colors="neutral")


class RunCommand(BaseCommand):
    """
    Run pomace in a loop

    run
        {--browser= : Browser to use for automation}
        {--headless : Run the specified browser in a headless mode}
        {--domain= : Starting domain for the automation}
        {--root= : Directory to load models from}
    """

    RELOAD_ACTIONS = "<reload actions>"

    def run_loop(self):
        self.clear_screen()
        page = models.autopage()
        self.display_url(page)
        while True:
            choices = [self.RELOAD_ACTIONS] + dir(page)
            cli = bullet.Bullet(
                prompt="\nSelect an action: ", bullet=" ● ", choices=choices
            )
            action = cli.launch()
            if action == self.RELOAD_ACTIONS:
                reload(models)
                self.clear_screen()
                page = models.autopage()
                self.display_url(page)
                continue

            cli = bullet.Input(prompt="\nValue: ")
            page, transitioned = page.perform(action, prompt=cli.launch)
            if transitioned:
                self.clear_screen()
                self.display_url(page)

    def clear_screen(self):
        os.system("cls" if os.name == "nt" else "clear")

    def display_url(self, page):
        self.line(f"<fg=white;options=bold>{page}</>")


application = Application()
application.add(ShellCommand())
application.add(RunCommand())



# pylint: disable=expression-not-assigned,unused-variable,redefined-outer-name,unused-argument

import pytest

from .. import utils


def describe_fake():
    @pytest.fixture(scope='session')
    def fake():
        return utils.Fake()

    def describe_person():
        def it_includes_name_in_email(expect, fake):
            person = fake.person
            expect(person.email).icontains(person.last_name)

        def it_includes_honorific(expect, fake):
            expect(fake.person.honorific).is_not(None)



from contextlib import suppress
from typing import Callable, List, Optional, Tuple

import log
from bs4 import BeautifulSoup
from datafiles import datafile, field, mapper
from selenium.common.exceptions import WebDriverException
from splinter.driver.webdriver import WebDriverElement
from splinter.exceptions import ElementDoesNotExist

from . import shared
from .config import settings
from .enums import Mode, Verb
from .types import URL


__all__ = ['Locator', 'Action', 'Page']


@datafile(order=True)
class Locator:

    mode: str = field(default='', compare=False)
    value: str = field(default='', compare=False)
    index: int = field(default=0, compare=False)
    score: float = field(default=0.5, compare=True)

    @property
    def _mode(self) -> Mode:
        return Mode(self.mode)

    def __bool__(self) -> bool:
        return bool(self.mode and self.value)

    def find(self) -> Optional[WebDriverElement]:
        elements = self._mode.finder(self.value)
        try:
            element = elements[self.index]
        except ElementDoesNotExist:
            log.debug(f'{self} unable to find element')
            return None
        else:
            log.debug(f'{self} found element: {element}')
            return element

    def increase_score(self):
        if self.score:
            new_score = round(min(1.0, self.score * 1.25), 4)
        else:
            new_score = 0.1
        if new_score > self.score:
            log.debug(f'Increasing score of {self} to {new_score}')
            self.score = new_score

    def decrease_score(self):
        new_score = round(self.score * 0.5, 4)
        if new_score < self.score:
            log.debug(f'Decreasing score of {self} to {new_score}')
            self.score = new_score


@datafile
class Action:

    verb: str = ''
    name: str = ''
    locators: List[Locator] = field(default_factory=lambda: [Locator()])

    @property
    def _verb(self) -> Verb:
        return Verb(self.verb)

    def __post_init__(self):
        if self.verb and len(self.locators) <= 1:
            for mode, value in self._verb.get_default_locators(self.name):
                self.locators.append(Locator(mode, value))

    def __str__(self):
        return f'{self.verb}_{self.name}'

    def __bool__(self) -> bool:
        return bool(self.verb and self.name)

    def __call__(self, *args, **kwargs) -> 'Page':
        page = kwargs.pop('_page', None)
        page = self._call_method(page, *args, **kwargs)
        self.datafile.save()
        return page

    def _call_method(self, page: Optional['Page'], *args, **kwargs) -> 'Page':
        for locator in sorted(self.locators, reverse=True):
            if locator:
                log.debug(f'Using {locator} to find {self.name!r}')
                element = locator.find()
                if element:
                    if self._perform_action(element, *args, **kwargs):
                        locator.increase_score()
                        break
            locator.decrease_score()
        else:
            log.error(f'No locators able to find {self.name!r}')
            if page:
                return page

        if page and self._verb.updates:
            return page

        return autopage()

    def _perform_action(self, element: WebDriverElement, *args, **kwargs) -> bool:
        delay = kwargs.pop('delay', 0.0)
        function = getattr(element, self.verb)
        try:
            function(*args, **kwargs)
        except WebDriverException as e:
            log.debug(e)
            return False
        else:
            self._verb.post_action(delay=delay)
            return True


@datafile(
    "./sites/{self.domain}/{self.path}/{self.variant}.yml", defaults=True, manual=True
)
class Page:

    domain: str
    path: str = URL.ROOT
    variant: str = 'default'

    active_locators: List[Locator] = field(default_factory=lambda: [Locator()])
    inactive_locators: List[Locator] = field(default_factory=lambda: [Locator()])

    actions: List[Action] = field(default_factory=lambda: [Action()])

    @classmethod
    def at(cls, url: str, *, variant: str = '') -> 'Page':
        if shared.browser.url != url:
            log.info(f"Visiting {url}")
            shared.browser.visit(url)

        if shared.browser.url != url:
            log.info(f"Redirected to {url}")

        kwargs = {'domain': URL(url).domain, 'path': URL(url).path}
        variant = variant or URL(url).fragment
        if variant:
            kwargs['variant'] = variant

        return cls(**kwargs)  # type: ignore

    @property
    def url(self) -> URL:
        return URL(self.domain, self.path)

    @property
    def active(self) -> bool:
        log.debug(f'Determining if {self!r} is active')

        if self.url != URL(shared.browser.url):
            log.debug(
                f'{self!r} is inactive - URL does not match: {shared.browser.url}'
            )
            return False

        log.debug('Checking that all expected elements can be found')
        for locator in self.active_locators:
            if locator and not locator.find():
                log.debug(f'{self!r} is inactive - Unable to find: {locator!r}')
                return False

        log.debug('Checking that no unexpected elements can be found')
        for locator in self.inactive_locators:
            if locator and locator.find():
                log.debug(f'{self!r} is inactive - Found unexpected: {locator!r}')
                return False

        log.debug(f'{self!r} is active')
        return True

    @property
    def text(self) -> str:
        return shared.browser.html

    @property
    def html(self) -> BeautifulSoup:
        return BeautifulSoup(self.text, 'html.parser')

    def __repr__(self):
        if self.variant == 'default':
            return f"Page.at('{self.url.value}')"
        return f"Page.at('{self.url.value}', variant='{self.variant}')"

    def __str__(self):
        if self.variant == 'default':
            return f'{self.url}'
        return f'{self.url} ({self.variant})'

    def __dir__(self):
        names = []
        add_placeholder = True
        for action in self.actions:
            if action:
                names.append(str(action))
            else:
                add_placeholder = False
        if add_placeholder:
            self.actions.append(Action())
        return names

    def __getattr__(self, value: str) -> Action:
        if '_' in value:
            verb, name = value.split('_', 1)

            with suppress(FileNotFoundError):
                self.datafile.load()

            for action in self.actions:
                if action.name == name and action.verb == verb:
                    return action

            if Verb.validate(verb):
                action = Action(verb, name)
                setattr(
                    action, 'datafile', mapper.create_mapper(action, root=self.datafile)
                )
                self.actions.append(action)
                return action

        return object.__getattribute__(self, value)

    def __contains__(self, value):
        return value in self.text

    def perform(self, name: str, *, prompt: Callable) -> Tuple['Page', bool]:
        action = getattr(self, name)
        if action.verb in {'fill', 'select'}:
            value = settings.get_secret(action.name) or prompt()
            settings.update_secret(action.name, value)
            page = action(value, _page=self)
        else:
            page = action(_page=self)
        return page, page != self


def autopage() -> Page:
    matching_pages = []

    for page in Page.objects.filter(domain=URL(shared.browser.url).domain):
        if page.active:
            matching_pages.append(page)

    if matching_pages:
        if len(matching_pages) > 1:
            for page in matching_pages:
                log.warn(f'Multiple pages matched: {page}')
        return matching_pages[0]

    log.info(f'Creating new page: {shared.browser.url}')
    page = Page.at(shared.browser.url)
    page.datafile.save()
    return page



# pylint: disable=expression-not-assigned,unused-variable,redefined-outer-name,unused-argument

import pytest

from ..models import Action, Locator, Page


@pytest.fixture
def locator():
    return Locator('name', 'email')


@pytest.fixture
def action():
    return Action('fill', 'email')


@pytest.fixture
def page():
    return Page('example.com', '/foo/bar')


def describe_locator():
    def describe_bool():
        def it_is_false_when_placeholder(expect, locator):
            expect(bool(locator)) == True
            locator.value = ''
            expect(bool(locator)) == False

    def describe_sort():
        def it_orders_by_score(expect):
            locators = [
                Locator('name', 'bbb', score=0.5),
                Locator('name', 'aaa', score=0.6),
                Locator('name', 'BBB', score=0.6),
                Locator('name', 'AAA', score=0.7),
                Locator('name', 'zzz', score=0.8),
            ]
            expect(sorted(locators)) == locators

    def describe_find():
        def it_returns_callable(expect, mockbrowser, locator):
            expect(locator.find()) == '<mockelement: name=email>'

        def it_can_find_links_by_partial_text(expect, mockbrowser, locator):
            locator.mode = 'partial_text'
            expect(locator.find()) == '<mockelement: links.partial_text=email>'


def describe_action():
    def describe_str():
        def it_includes_the_verb_and_name(expect, action):
            expect(str(action)) == 'fill_email'

    def describe_bool():
        def it_is_false_when_placeholder(expect, action):
            expect(bool(action)) == True
            action.name = ''
            expect(bool(action)) == False


def describe_page():
    def describe_repr():
        def it_includes_the_url(expect, page):
            expect(repr(page)) == "Page.at('https://example.com/foo/bar')"

        def it_includes_the_variant_when_set(expect, page):
            page.variant = 'qux'
            expect(
                repr(page)
            ) == "Page.at('https://example.com/foo/bar', variant='qux')"

    def describe_str():
        def it_returns_the_url(expect, page):
            expect(str(page)) == 'https://example.com/foo/bar'

        def it_includes_the_variant_when_set(expect, page):
            page.variant = 'qux'
            expect(str(page)) == 'https://example.com/foo/bar (qux)'

    def describe_dir():
        def it_lists_valid_actions(expect, page, action):
            page.actions = []
            expect(dir(page)) == []
            page.actions.append(action)
            expect(dir(page)) == ['fill_email']

    def describe_getattr():
        def it_returns_matching_action(expect, page, action):
            page.actions = [action]
            expect(getattr(page, 'fill_email')) == action

        def it_adds_missing_actions(expect, page, monkeypatch):
            page.actions = []
            new_action = getattr(page, 'fill_password')
            expect(new_action.verb) == 'fill'
            expect(new_action.name) == 'password'
            expect(len(new_action.locators)) > 1

        def it_rejects_invalid_actions(expect, page):
            with expect.raises(AttributeError):
                getattr(page, 'mash_password')

        def it_rejects_missing_attributes(expect, page):
            with expect.raises(AttributeError):
                getattr(page, 'foobar')

    def describe_contains():
        def it_matches_partial_html(expect, page, mockbrowser):
            expect(page).contains("world")



# pylint: disable=expression-not-assigned,unused-variable,redefined-outer-name,unused-argument

import pytest

from ..types import URL


@pytest.fixture
def url():
    return URL('http://www.example.com/foo/bar')


def describe_url():
    def describe_init():
        def with_url(expect, url):
            expect(url.value) == 'http://www.example.com/foo/bar'

        def with_url_and_trailing_slash(expect):
            url = URL('http://example.com/login/error/')
            expect(url.value) == 'http://example.com/login/error'

        def with_path(expect):
            url = URL('example.com', 'login')
            expect(url.value) == 'https://example.com/login'

        def with_path_at_root(expect):
            url = URL('example.com', '@')
            expect(url.value) == 'https://example.com'

        def with_path_and_extra_slashes(expect):
            url = URL('example.com', '/login/error/')
            expect(url.value) == 'https://example.com/login/error'

    def describe_str():
        def it_returns_the_value(expect, url):
            expect(str(url)) == 'http://www.example.com/foo/bar'

    def describe_eq():
        def it_compares_domain_and_path(expect, url):
            expect(url) == URL('https://www.example.com/foo/bar/')
            expect(url) != URL('http://example.com/foo/bar')

        def it_matches_patterns(expect):
            pattern = URL('http://example.com/p/{name}')
            expect(pattern) == URL('http://example.com/p/foobar')
            expect(pattern) != URL('http://example.com/')
            expect(pattern) != URL('http://example.com/p/foo/bar')

        def it_can_be_compared_to_str(expect, url):
            expect(url) == str(url)
            expect(url) == str(url) + '/'
            expect(url) != str(url) + '_extra'

    def describe_contains():
        def it_checks_url_contents(expect, url):
            expect(url).contains("foo/bar")
            expect(url).excludes("qux")

    def describe_path():
        def when_root(expect, url):
            url = URL('http://example.com')
            expect(url.path) == '@'

        def when_single(expect, url):
            url = URL('http://example.com/login')
            expect(url.path) == 'login'

        def when_trailing_slash(expect, url):
            url = URL('http://example.com/login/error/')
            expect(url.path) == 'login/error'

    def describe_fragment():
        def when_default(expect, url):
            expect(url.fragment) == ''

        def when_provided(expect, url):
            url = URL('http://example.com/signup/#step1')
            expect(url.fragment) == 'step1'

        def when_containing_slashes(expect, url):
            url = URL('http://example.com/signup/#/step/2/')
            expect(url.fragment) == 'step_2'



# pylint: disable=expression-not-assigned,unused-variable,redefined-outer-name,unused-argument

from ..enums import Verb


def describe_verb():
    def describe_get_default_locators():
        def when_click(expect):
            verb = Verb('click')
            locators = list(verb.get_default_locators('send_email'))
            expect(locators) == [('text', "Send Email"), ('text', "Send email")]

        def when_fill(expect):
            verb = Verb('fill')
            locators = list(verb.get_default_locators('first_name'))
            expect(locators) == [
                ('name', "first_name"),
                ('name', "first-name"),
                ('id', "first_name"),
                ('id', "first-name"),
                ('css', '[aria-label="First Name"]'),
                ('id', "FirstName"),
            ]



import inspect
import os
import random
import time
from pathlib import Path

import faker
import log

from . import browser, shared
from .config import settings
from .types import Person


class Fake:
    def __init__(self):
        self.generator = faker.Faker()

    def __getattr__(self, name):
        method = getattr(self.generator, name)
        return method()

    @property
    def person(self) -> Person:
        if random.random() > 0.5:
            prefix, first_name, last_name = (
                self.prefix_female,
                self.first_name_female,
                self.last_name,
            )
        else:
            prefix, first_name, last_name = (
                self.prefix_male,
                self.first_name_male,
                self.last_name,
            )
        email_address = f'{first_name}{last_name}@{self.free_email_domain}'.lower()
        return Person(prefix, first_name, last_name, email_address, self.postcode)


def launch_browser(delay: float = 0.0) -> bool:
    launched = False
    if not shared.browser:
        shared.browser = browser.launch()
        browser.resize(shared.browser)
        launched = True
    shared.browser.visit(settings.url)
    time.sleep(delay)
    return launched


def quit_browser(silence_logging: bool = False):
    if silence_logging:
        log.silence('pomace', 'selenium', allow_warning=True)
    try:
        browser.save_url(shared.browser)
        browser.save_size(shared.browser)
        shared.browser.quit()
    except Exception as e:
        log.debug(e)


def locate_models(*, caller=None):
    cwd = Path.cwd()

    if caller:
        for frame in inspect.getouterframes(caller):
            if 'pomace' not in frame.filename or 'pomace/tests' in frame.filename:
                path = Path(frame.filename)
                log.debug(f"Found caller's package directory: {path.parent}")
                os.chdir(path.parent)
                return

    if (cwd / 'sites').is_dir():
        log.debug(f"Found models in current directory: {cwd}")
        return

    for path in cwd.iterdir():
        if (path / 'sites').is_dir():
            log.debug(f"Found models in package directory: {path}")
            os.chdir(path)
            return



import atexit
import inspect
from typing import Optional

import log

from . import cli, models, utils
from .config import settings


__all__ = ['fake', 'prompt', 'visit']


fake = utils.Fake()


def visit(
    url: str = '',
    *,
    browser: str = '',
    headless: Optional[bool] = None,
    delay: float = 0.0,
) -> models.Page:
    if url:
        settings.url = url
    else:
        cli.prompt_for_url_if_unset()

    if browser:
        settings.browser.name = browser.lower()
    else:
        cli.prompt_for_browser_if_unset()

    if headless is not None:
        settings.browser.headless = headless

    if utils.launch_browser(delay):
        log.silence('urllib3.connectionpool')
        atexit.register(utils.quit_browser, silence_logging=True)

    utils.locate_models(caller=inspect.currentframe())

    return models.autopage()


def prompt(*names):
    for name in names:
        cli.prompt_for_secret_if_unset(name)
    return settings



import time
from enum import Enum
from typing import Iterator, Optional, Tuple

import inflection
from selenium.webdriver.common.keys import Keys

from . import shared


class Mode(Enum):

    CSS = 'css'
    XPATH = 'xpath'
    TAG = 'tag'
    NAME = 'name'
    TEXT = 'text'
    PARTIAL_TEXT = 'partial_text'
    ID = 'id'
    VALUE = 'value'

    @property
    def finder(self):
        if self is self.PARTIAL_TEXT:
            return getattr(shared.browser.links, f'find_by_{self.value}')
        return getattr(shared.browser, f'find_by_{self.value}')


class Verb(Enum):
    CLICK = 'click'
    FILL = 'fill'
    SELECT = 'select'
    CHOOSE = 'choose'

    @classmethod
    def validate(cls, value: str) -> bool:
        return value in [e.value for e in cls]

    @property
    def updates(self) -> bool:
        return self not in {self.CLICK}

    @property
    def delay(self) -> float:
        if self in {self.CLICK}:
            return 1.0
        return 0.0

    def get_default_locators(self, name: str) -> Iterator[Tuple[str, str]]:
        if self is self.CLICK:
            yield Mode.TEXT.value, inflection.titleize(name)
            yield Mode.TEXT.value, inflection.humanize(name)
        elif self is self.FILL:
            yield Mode.NAME.value, name
            yield Mode.NAME.value, inflection.dasherize(name)
            yield Mode.ID.value, name
            yield Mode.ID.value, inflection.dasherize(name)
            yield Mode.CSS.value, f'[aria-label="{inflection.titleize(name)}"]'
            yield Mode.ID.value, inflection.titleize(name).replace(' ', '')

    def post_action(self, *, delay: Optional[float] = None):
        if delay is None:
            delay = self.delay
        if self is self.FILL:
            element = shared.browser.driver.switch_to.active_element
            element.send_keys(Keys.TAB)
        time.sleep(delay)



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['pomace', 'pomace.tests']

package_data = \
{'': ['*']}

install_requires = \
['beautifulsoup4>=4.8.2,<5.0.0',
 'bullet>=2.1.0,<3.0.0',
 'cleo>=0.7.6,<0.8.0',
 'datafiles>=0.10,<0.11',
 'faker>=4.1.1,<5.0.0',
 'inflection>=0.4.0,<0.5.0',
 'ipython>=7.12.0,<8.0.0',
 'minilog>=1.6,<2.0',
 'parse>=1.14.0,<2.0.0',
 'splinter>=0.12.0,<0.13.0',
 'webdriver_manager>=2.5.0,<3.0.0']

entry_points = \
{'console_scripts': ['pomace = pomace.cli:application.run']}

setup_kwargs = {
    'name': 'pomace',
    'version': '0.3b6',
    'description': 'Dynamic page objects for browser automation.',
    'long_description': '# Overview\n\nDynamic page objects for browser automation.\n\nThis project was generated with [cookiecutter](https://github.com/audreyr/cookiecutter) using [jacebrowning/template-python](https://github.com/jacebrowning/template-python).\n\n[![Unix Build Status](https://img.shields.io/travis/jacebrowning/pomace/develop.svg?label=unix)](https://travis-ci.org/jacebrowning/pomace)\n[![Windows Build Status](https://img.shields.io/appveyor/ci/jacebrowning/pomace/develop.svg?label=window)](https://ci.appveyor.com/project/jacebrowning/pomace)\n[![Coverage Status](https://img.shields.io/coveralls/jacebrowning/pomace/develop.svg)](https://coveralls.io/r/jacebrowning/pomace)\n[![PyPI Version](https://img.shields.io/pypi/v/pomace.svg)](https://pypi.org/project/pomace)\n[![PyPI License](https://img.shields.io/pypi/l/pomace.svg)](https://pypi.org/project/pomace)\n\n# Setup\n\n## Requirements\n\n- Python 3.7+\n\n## Installation\n\nOn macOS:\n\n```\n$ brew install pipx\n$ pipx ensurepath\n$ pipx install pomace\n$ pomace shell\n$ pomace run\n```\n',
    'author': 'Jace Browning',
    'author_email': 'jacebrowning@gmail.com',
    'maintainer': None,
    'maintainer_email': None,
    'url': 'https://pypi.org/project/pomace',
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'entry_points': entry_points,
    'python_requires': '>=3.7,<4.0',
}


setup(**setup_kwargs)



import sys

import log
from splinter import Browser
from splinter.exceptions import DriverNotFoundError
from webdriver_manager import chrome, firefox

from .config import settings


NAMES = ['Firefox', 'Chrome']

WEBDRIVER_MANAGERS = {
    "chromedriver": chrome.ChromeDriverManager,
    "geckodriver": firefox.GeckoDriverManager,
}


def launch() -> Browser:
    options = {"headless": settings.browser.headless, "wait_time": 1.0}

    if not settings.browser.name:
        sys.exit("No browser specified")

    # TODO: Only log when not IPython
    # log.info(f'Launching browser: {settings.browser.name}')

    try:
        return Browser(settings.browser.name, **options)
    except DriverNotFoundError:
        sys.exit(f"Unsupported browser: {settings.browser.name}")
    except Exception as e:  # pylint: disable=broad-except
        log.debug(str(e))

        for driver, manager in WEBDRIVER_MANAGERS.items():
            if driver in str(e).lower():
                options["executable_path"] = manager().install()
                return Browser(settings.browser.name, **options)

        raise e from None


def resize(browser: Browser):
    browser.driver.set_window_size(settings.browser.width, settings.browser.height)
    browser.driver.set_window_position(0, 0)
    size = browser.driver.get_window_size()
    log.debug(f'Resized browser: {size}')


def save_url(browser: Browser):
    if settings.browser != browser.url:
        log.debug(f"Saving last browser url: {browser.url}")
        settings.url = browser.url


def save_size(browser: Browser):
    size = browser.driver.get_window_size()
    if size != (settings.browser.width, settings.browser.height):
        log.debug(f'Saving last browser size: {size}')
        settings.browser.width = size['width']
        settings.browser.height = size['height']
